#include <stdio.h>
#include <math.h>
#include "common.h"
#include "cluster.h"
#include "mpi.h"


/*--------------------------------------------------------------------

  Copy label and bond edge values between neighboring processors.

----------------------------------------------------------------------*/

void bond_edges()
{
  int type, len,vele,voffset,items;
  int  i,j,k;
  MPI_Status status;
  MPI_Datatype strideType;



  type = EDGE;


  /* top edge -> bottom edge  (bottom node passes to top node). */

  vele    = 1;
  voffset = MAX_BUFF;
  items   = isiz;

  MPI_Type_vector(items, vele, voffset, MPI_INT, &strideType);

  /* New data type must be committed before it can be used */
  MPI_Type_commit(&strideType);

  MPI_Sendrecv(&jbond[1][jsiz], vele, strideType, top,    type,
               &jbond[1][0],    vele, strideType, bottom, type,
               comm2d, &status);

  MPI_Type_free(&strideType);




  /* right edge -> left edge  (left node passes to right node). */

  len = jsiz;

  MPI_Sendrecv(&ibond[isiz][1], len, MPI_INT, right, type,
	       &ibond[0][1],    len, MPI_INT, left,  type,
	       comm2d, &status);
 

      
};
 



/*==================================================================*/

void get_edges()
{

  int type; 
  int len1,len2;
  int i,j,k;
  MPI_Status status;


  /* Pack first cluster then new spin into sendi & sendj. */
  /* Cluster labels: */
  for (i=1; i<=isiz; i++)
    sendi[i] = Npoint[label[i][1]];

  for (j=1; j<=jsiz; j++)
    sendj[j] = Npoint[label[1][j]];


  /* New spins: */
  i = isiz + 1;
  for (k=1; k<=isiz; k++) {
    sendi[i] = new_spin[label[k][1]];
    i++;
  }

  j = jsiz + 1;
  for (k=1; k<=jsiz; k++) {
    sendj[j] = new_spin[label[1][k]];
    j++;
  }



  /* Get cluster and new_spin edges. */
  type = EDGE;

  /* left edge -> right edge  (right node passes to left node). */
	
  len1 = jsiz*2;

  MPI_Sendrecv(&sendj[1], len1, MPI_INT, left,  type,
	       &recvj[1], len1, MPI_INT, right, type,
	       comm2d, &status);



  /* bottom edge -> top edge  (top node passes to bottom node). */

  len2 = isiz*2;

  MPI_Sendrecv(&sendi[1], len2, MPI_INT, bottom,  type,
	       &recvi[1], len2, MPI_INT, top,     type,
	       comm2d, &status);



};



/*================================================================*/


void return_edges()
{
  int type; 
  int len1,len2;
  int i,j,k;
  MPI_Status status;


  /* Pack first cluster then new spin into sendi & sendj. */
  /* Cluster labels: */
  for (i=1; i<=isiz; i++)
    sendi[i] = Npoint[label[i][jsiz]];

  for (j=1; j<=jsiz; j++)
    sendj[j] = Npoint[label[isiz][j]];


  /* New spins: */
  i = isiz + 1;
  for (k=1; k<=isiz; k++) {
    sendi[i] = new_spin[label[k][jsiz]];
    i++;
  }

  j = jsiz + 1;
  for (k=1; k<=jsiz; k++) {
    sendj[j] = new_spin[label[isiz][k]];
    j++;
  }



  /* Return cluster and new_spin edges. */
  type = EDGE;

  /* right edge -> left edge  (left node passes to right node). */

  len1 = jsiz*2;

  MPI_Sendrecv(&sendj[1], len1, MPI_INT, right, type,
	       &recvj[1], len1, MPI_INT, left,  type,
	       comm2d, &status);


  /* top edge -> bottom edge  (bottom node passes to top node). */

  len2 = isiz*2;

  MPI_Sendrecv(&sendi[1], len2, MPI_INT, top,    type,
	       &recvi[1], len2, MPI_INT, bottom, type,
	       comm2d, &status);



};
      



/*=================================================================*/


void label_edges()
{

  int type, i,j, len1,len2;
  MPI_Status status;


  for (i=1; i<=isiz; i++)
    sendi[i] = label[i][1];

  for (j=1; j<=jsiz; j++)
    sendj[j] = label[1][j];



  type = EDGE;

  /* left edge -> right edge  (right node passes to left node). */

  len1 = jsiz;

  MPI_Sendrecv(&sendj[1], len1, MPI_INT, left,  type,
               &recvj[1], len1, MPI_INT, right, type,
               comm2d, &status);



  /* bottom edge -> top edge  (top node passes to bottom node). */

  len2 = isiz;

  MPI_Sendrecv(&sendi[1], len2, MPI_INT, bottom,  type,
               &recvi[1], len2, MPI_INT, top,     type,
               comm2d, &status);



  for (i=1; i<=isiz; i++)
    label[i][jsiz+1] = recvi[i];
  
  for (j=1; j<=jsiz; j++) 
    label[isiz+1][j] = recvj[j];




};

