#include "common.h"
#include "cluster.h"
#include "mpi.h"

/*---------------------------------------------------------------------
                                                                      
     Local label propagation (or "self-labeling") to do               
     global cluster labeling across processors.                      
                                                                      
---------------------------------------------------------------------*/



void  global_label()
{
  int i,j,labeli,labeln,type,bytes;
  int changed, sendchanged, recvchanged, how_many;


  how_many = 0;

  /* Loop until nothing changes. */
  while(1) {
    changed = 0;

    /* Get cluster labels and new spins from processors 
       in +ve i and j directions. */
    get_edges();	 


    /* i edge. */
    j = jsiz;
    for (i=1; i<=isiz; i++) {
      labeli = Npoint[label[i][j]];
      if (jbond[i][j]) {
	labeln = recvi[i];
	if (labeln < labeli) {
	  Npoint[label[i][j]] = labeln;
	  new_spin[label[i][j]] = recvi[i+isiz];
	  changed = 1;
	}
	else
	  if (labeln > labeli) changed = 1;
	/* Don't do anything since will send back Npoint(label(i,j))
	   and new_spin(label(i,j)), which will be different.
	   else labeli = labeln so nothing changes. */
      }        
    }


    /* j edge. */
    i = isiz;
    for (j=1; j<=jsiz; j++) {
      labeli = Npoint[label[i][j]];
      if (ibond[i][j]) {
	labeln = recvj[j];
	if (labeln < labeli) {
	  Npoint[label[i][j]] = labeln;
	  new_spin[label[i][j]] = recvj[j+jsiz];
	  changed = 1;
	}
	else
	  if (labeln > labeli) changed = 1;
	/*  Don't do anything since will send back Npoint(label(i,j)) 
	    and new_spin(label(i,j)), which will be different.
	    else labeli = labeln so nothing changes. */
      }
    }


    /* Return cluster labels and new spins to processors 
    in +ve i and j dirs. */
    return_edges();


    /* Unpack first cluster then spin from recvi & recvj, and
    update N and new_spin if the new cluster labels are smaller. */
    j = 1;
    for (i=1; i<=isiz; i++) {
      if (jbond[i][0] && (recvi[i] < Npoint[label[i][j]])) {
	Npoint[label[i][j]] = recvi[i];
	new_spin[label[i][j]] = recvi[i+isiz];
      }
    }
    i = 1;
    for (j=1; j<=jsiz; j++) {
      if (ibond[0][j] && (recvj[j] < Npoint[label[i][j]])) {
	Npoint[label[i][j]] = recvj[j];
	new_spin[label[i][j]] = recvj[j+jsiz]; 
      }    
    }

    /* Test for termination by combining changed's. Only start testing 
    after minits iterations, to save doing unnecessary broadcasts. */
    if (how_many >= minits) {
      sendchanged = changed;
      MPI_Allreduce(&sendchanged, &recvchanged, 1, MPI_INT,
		    MPI_SUM, comm2d);
      changed = recvchanged;

    }
    else
      changed = 1;


    how_many++;

    
    if (changed <= 0)   break;
    
  
  } /* end while */


   ACCEPT = how_many * globsites;



};



