/*
 * File         : Datatype.java
 * Author       : Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/04/09 18:39:45 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

package mpi;
import mpi.*;

public class Datatype {
  public final static int NULL    = 0;
  public final static int BYTE    = 1;
  public final static int CHAR    = 2;
  public final static int SHORT   = 3;
  public final static int BOOLEAN = 4;
  public final static int INT     = 5;
  public final static int LONG    = 6;
  public final static int FLOAT   = 7;
  public final static int DOUBLE  = 8;  
  public final static int PACKED  = 9;
  public final static int LB      =10;
  public final static int UB      =11;

  private static native void init();

  public Datatype(int Type) 
  { GetDatatype(Type); baseType = this; 
	baseTypeInd=Type;}

  private Datatype(long _handle, Datatype base) 
  { handle = _handle; baseType = base.baseType;
	baseTypeInd=base.baseTypeInd; }
  private native void GetDatatype(int Type);

  public native int Extent();
  public native int Size();
  public native int Lb();
  public native int Ub();
  public native void Commit();

  public native void finalize();

  public int baseExtent() { return baseType.Extent();}
  public int baseSize() { return baseType.Size();}

  public static Datatype Contiguous(int count, Datatype oldtype) 
  { return new Datatype(GetContiguous(count, oldtype),oldtype);}
  public static Datatype Vector(int count, int blocklength, int stride, Datatype oldtype)
  { return new Datatype(GetVector(count, blocklength, stride,oldtype), oldtype);}
  public static Datatype Hvector(int count, int blocklength, int stride, Datatype oldtype)
  { return new Datatype(GetHvector(count, blocklength, stride,oldtype), oldtype);}
  public static Datatype Indexed(int[] blocklengths, 
    int[] displacements, Datatype oldtype)
  { return new Datatype(GetIndexed(blocklengths,
    displacements,oldtype), oldtype);}
  public static Datatype Hindexed(int[] blocklengths, 
    int[] displacements, Datatype oldtype)
  { return new Datatype(GetHindexed(blocklengths,
    displacements,oldtype), oldtype);}

  public static Datatype Struct(int[] blocklengths,   
    int[] displacements, Datatype[] types) 
  { return new Datatype(GetStruct(blocklengths, 
	displacements, types), types[0]);
	/*
    handle = GetStruct(count, blocklengths, displacements, types);
    baseType = types[0];
    */
  }

  private static native long GetContiguous(int count,Datatype oldtype);
  private static native long GetVector(int count,int blocklength, int stride,Datatype oldtype);
  private static native long GetHvector(int count,int blocklength, int stride,Datatype oldtype);
  private static native long GetIndexed(int[] blocklengths, 
    int[] displacements,Datatype oldtype);
  private static native long GetHindexed(int[] blocklengths, 
    int[] displacements,Datatype oldtype);  
  private static native long GetStruct(int[] blocklengths, 
    int[] displacements, Datatype[] types);  

  protected long handle;
  protected Datatype baseType;
  protected int baseTypeInd;

  static {
    init();
  }

}

