/*
 * File         : mpi_Comm.c
 * Headerfile   : mpi_Comm.h 
 * Author       : Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.17 $
 * Updated      : $Date: 1998/04/09 18:39:53 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <mpi.h>
#include "mpi_Comm.h"

jfieldID CommhandleID;
extern jfieldID reqhandleID, DatatypehandleID, ErrhandleID, GrouphandleID;
extern jfieldID sourceID,tagID,countID, indexID;
/* extern jfieldID errorID; */

/*
 * Class:     mpi_Comm
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_init
  (JNIEnv *env, jclass thisClass)
{
  CommhandleID = (*env)->GetFieldID(env,thisClass,"handle","J");
}

/*
 * Class:     mpi_Comm
 * Method:    GetComm
 * Signature: (I)J
 */
JNIEXPORT void JNICALL Java_mpi_Comm_GetComm
  (JNIEnv *env, jobject jthis, jint type)
{
  switch (type) {
    case 0:
	  (*env)->SetLongField(env,jthis, CommhandleID,(jlong)MPI_COMM_NULL);
      break;
    case 1:
	  (*env)->SetLongField(env,jthis, CommhandleID,(jlong)MPI_COMM_SELF); 
      break;
    case 2:
	  (*env)->SetLongField(env,jthis, CommhandleID,(jlong)MPI_COMM_WORLD);
      break;
  }
  /* printf("Comm_constructor ... \n"); */
}


/*
 * Class:     mpi_Comm
 * Method:    Size
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_Size
  (JNIEnv *env, jobject jthis)
{
  int size;
  MPI_Comm_size((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)), &size);
  return size;
}

/*
 * Class:     mpi_Comm
 * Method:    Rank
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_Rank
  (JNIEnv *env, jobject jthis) {
  int rank;
  MPI_Comm_rank((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)), &rank);
  return rank;
}

/*
 * Class:     mpi_Comm
 * Method:    Dup
 * Signature: ()Lmpi/Comm;
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_dup
  (JNIEnv *env, jobject jthis)
{
	MPI_Comm newcomm;
    MPI_Comm_dup((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)), &newcomm);
	return (jlong)newcomm;
}

/*
 * Class:     mpi_Comm
 * Method:    Compare
 * Signature: (Lmpi/Comm;Lmpi/Comm;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_Compare
  (JNIEnv *env, jclass jthis, jobject comm1, jobject comm2)
{
    int result;
    MPI_Comm_compare(
        (MPI_Comm)((*env)->GetLongField(env,comm1,CommhandleID)),
        (MPI_Comm)((*env)->GetLongField(env,comm2,CommhandleID)),
        &result);
    return result;
}

/*
 * Class:     mpi_Comm
 * Method:    Free, finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_finalize
  (JNIEnv *env, jobject jthis)
{
  MPI_Comm comm=(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID));
  MPI_Comm_free(&comm);
  (*env)->SetLongField(env,jthis, CommhandleID,0);
}

/*
 * Class:     mpi_Comm
 * Method:    group
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_group
  (JNIEnv *env, jobject jthis)
{
    MPI_Group group;
    MPI_Comm_group((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),
		&group);
    return (jlong)group;
}

/*
 * Class:     mpi_Comm
 * Method:    Test_inter
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_mpi_Comm_Test_1inter
  (JNIEnv *env, jobject jthis)
{
    int flag;
    MPI_Comm_test_inter((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&flag);
    if(flag==0)
        return JNI_FALSE;
    else
        return JNI_TRUE;
}

/*
 * Class:     mpi_Comm
 * Method:    GetIntercomm
 * Signature: (Lmpi/Comm;III)J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_GetIntercomm
  (JNIEnv *env, jobject jthis, jobject local_comm, jint local_leader, jint remote_leader, jint tag)
{
  MPI_Comm newintercomm;
  MPI_Intercomm_create((MPI_Comm)((*env)->GetLongField(env,local_comm,CommhandleID)),local_leader,
	(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)), remote_leader, tag, &newintercomm);
  return (jlong)newintercomm;
}

/* getBufPtr is used in 
   Send, Recv ...
   for getting pointer from 'jobject buf'
   Xinying Li 12/15/97
*/
jbyte *getBufPtr(JNIEnv *env,jobject buf, jobject type)
{
  jbyte *bufptr;
  jboolean isCopy=JNI_TRUE;
  jfieldID baseTypeIndID;
  jclass typeClass=(*env)->FindClass(env,"mpi/Datatype");
  baseTypeIndID= (*env)->GetFieldID(env,typeClass,"baseTypeInd","I");
  switch ((*env)->GetIntField(env,type,baseTypeIndID)) {
    case 0:
      break;
    case 1:
      bufptr=(*env)->GetByteArrayElements(env,buf,&isCopy);
      break;
    case 2:
      bufptr=(jbyte*)((*env)->GetCharArrayElements(env,buf,&isCopy));
      break;
    case 3:
      bufptr=(jbyte*)(*env)->GetShortArrayElements(env,buf,&isCopy);
      break;
    case 4:
      bufptr=(jbyte*)(*env)->GetBooleanArrayElements(env,buf,&isCopy);
      break;
    case 5:
      bufptr=(jbyte*)(*env)->GetIntArrayElements(env,buf,&isCopy);
      break;
    case 6:
      bufptr=(jbyte*)(*env)->GetLongArrayElements(env,buf,&isCopy);
      break;
    case 7:
      bufptr=(jbyte*)(*env)->GetFloatArrayElements(env,buf,&isCopy);
      break;
    case 8:
      bufptr=(jbyte*)(*env)->GetDoubleArrayElements(env,buf,&isCopy);
      break;
    case 9:
	  bufptr=(*env)->GetByteArrayElements(env,buf,&isCopy); 
      break;
    case 10:
      break;
	case 11:
      break;
    default:
      break;
  }
  return bufptr;
}

/* releaseBufPtr is used in 
   Send, Recv ...
   for release pointer corresponding to Get<...>ArrayElements
   Xinying Li 12/15/97
*/
void releaseBufPtr(JNIEnv *env,jobject buf,jobject type, jbyte* bufptr)
{
  jfieldID baseTypeIndID;
  jclass typeClass=(*env)->FindClass(env,"mpi/Datatype");
  baseTypeIndID= (*env)->GetFieldID(env,typeClass,"baseTypeInd","I");
  switch ((*env)->GetIntField(env,type,baseTypeIndID)) {
    case 0:
      break;
    case 1:
      (*env)->ReleaseByteArrayElements(env,buf,bufptr,0);
      break;
    case 2:
      (*env)->ReleaseCharArrayElements(env,buf,(jchar*)bufptr,0);
      break;
    case 3:
      (*env)->ReleaseShortArrayElements(env,buf,(jshort*)bufptr,0);
      break;
    case 4:
      (*env)->ReleaseBooleanArrayElements(env,buf,(jboolean*)bufptr,0);
      break;
    case 5:
      (*env)->ReleaseIntArrayElements(env,buf,(jint*)bufptr,0);
      break;
    case 6:
      (*env)->ReleaseLongArrayElements(env,buf,(jlong*)bufptr,0);
      break;
    case 7:
      (*env)->ReleaseFloatArrayElements(env,buf,(jfloat*)bufptr,0);
      break;
    case 8:
      (*env)->ReleaseDoubleArrayElements(env,buf,(jdouble*)bufptr,0);
      break;
    case 9:
	  (*env)->ReleaseByteArrayElements(env,buf,bufptr,0);
      break;
    case 10:
      break;
	case 11:
      break;
    default:
      break;
  }
}

/*
 * Class:     mpi_Comm
 * Method:    Send
 * Signature: (Ljava/lang/Object;IIILmpi/Datatype;II)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_Send
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint baseSize, jint
count, jobject type, jint dest, jint tag)
{
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Send(bufptr+offset*baseSize,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), dest, tag,
    (MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)));
  releaseBufPtr(env,buf,type,bufptr);
}

/*
 * Class:     mpi_Comm
 * Method:    Recv
 * Signature:
(Ljava/lang/Object;IILmpi/Datatype;IILmpi/Status;)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Recv
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint source, jint tag, jobject stat)
{
  MPI_Status status;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Recv(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	source, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&status);
/*  3/8 
  MPI_Get_count(&status, 
	(MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
    &status.count);
*/
  /* (*env)->SetIntField(env,stat, errorID,status.MPI_ERROR); */
  (*env)->SetIntField(env,stat, sourceID,status.MPI_SOURCE);
  (*env)->SetIntField(env,stat, tagID,status.MPI_TAG);
  (*env)->SetIntField(env,stat, countID,status.count);
  releaseBufPtr(env,buf,type,bufptr);
  return stat;
}

/*
 * Class:     mpi_Comm
 * Method:    Sendrecv
 * Signature:
(Ljava/lang/Object;IILmpi/Datatype;IILjava/lang/Object;IILmpi/Da
tatype;IILmpi/Status;)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Sendrecv
  (JNIEnv *env, jobject jthis, jobject sbuf, jint soffset, jint scount, 
	jobject stype, jint dest, jint stag, 
	jobject rbuf, jint roffset, jint rcount, 
	jobject rtype, jint source, jint rtag, jobject stat)
{
  MPI_Status status;
  jbyte * sbufptr=getBufPtr(env,sbuf,stype);
  jbyte * rbufptr=getBufPtr(env,rbuf,rtype);
  MPI_Sendrecv(sbufptr+soffset, scount, 
	(MPI_Datatype)((*env)->GetLongField(env,stype,DatatypehandleID)),
    dest, stag, rbufptr+roffset, rcount,
	(MPI_Datatype)((*env)->GetLongField(env,rtype,DatatypehandleID)),
	source, rtag,
	(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&status);
  releaseBufPtr(env,sbuf,stype,sbufptr);
  releaseBufPtr(env,rbuf,rtype,rbufptr);
/* 3/8
  MPI_Get_count(&status,
    (MPI_Datatype)((*env)->GetLongField(env,stype,DatatypehandleID)),
    &status.count);
*/
  (*env)->SetIntField(env,stat, sourceID,status.MPI_SOURCE);
  (*env)->SetIntField(env,stat, tagID,status.MPI_TAG);
  /* (*env)->SetIntField(env,stat, errorID,status.MPI_ERROR); */
  (*env)->SetIntField(env,stat, countID,status.count);
  return stat;
}

/*
 * Class:     mpi_Comm
 * Method:    Sendrecv_replace
 * Signature:
(Ljava/lang/Object;IILmpi/Datatype;IIILmpi/Datatype;IILmpi/Statu
s;)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Sendrecv_1replace
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count,
    jobject type, jint dest, jint stag,
    jint source, jint rtag, jobject stat)
{
  MPI_Status status;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Sendrecv_replace(bufptr+offset, count, 
    (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
    dest, stag, 
    source, rtag,
    (MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&status);
  releaseBufPtr(env,buf,type,bufptr);
/* 3/8
  MPI_Get_count(&status,
    (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
    &status.count);
*/
  (*env)->SetIntField(env,stat, sourceID,status.MPI_SOURCE);
  (*env)->SetIntField(env,stat, tagID,status.MPI_TAG);
  /* (*env)->SetIntField(env,stat, errorID,status.MPI_ERROR); */
  (*env)->SetIntField(env,stat, countID,status.count);
  return stat;
}

/* !! These Attr_put and Attr_get may be changed sometime later, because dbc
   suggest using NewGlobalRef.
   xli 1/23/98

   !! Attr_put and Attr_get are dealing with int attribute_val only now.
   xli 3/26/98
*/


/*/*
 * Class:     mpi_Comm
 * Method:    Attr_put
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_Attr_1put
  (JNIEnv *env, jobject jthis, jint keyval, jint attribute_val)
{
	MPI_Attr_put((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),
		     keyval, &attribute_val);
}

/*
 * Class:     mpi_Comm
 * Method:    Attr_get
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_Attr_1get
  (JNIEnv *env, jobject jthis, jint keyval)
{
    int *attribute_val; 
    int flag;
    MPI_Attr_get((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),
		 keyval, &attribute_val, &flag);
    return *attribute_val;
}

/*
 * Class:     mpi_Comm
 * Method:    Attr_put
 * Signature: (ILjava/lang/Object;)V
 */
/*
JNIEXPORT void JNICALL Java_mpi_Comm_Attr_1put
  (JNIEnv *env, jobject jthis, jint keyval, jobject attribute_val)
{
	MPI_Attr_put((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),keyval,
		attribute_val);
}
*/
/*
 * Class:     mpi_Comm
 * Method:    Attr_get
 * Signature: (I)Ljava/lang/Object;
 */
/*
JNIEXPORT jobject JNICALL Java_mpi_Comm_Attr_1get
  (JNIEnv *env, jobject jthis, jint keyval)
{
	jobject attribute_val;
	int flag;
    MPI_Attr_get((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),keyval,
        &attribute_val,&flag);
	return attribute_val;
}
*/

/*
 * Class:     mpi_Comm
 * Method:    Attr_delete
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_Attr_1delete
  (JNIEnv *env, jobject jthis, jint keyval)
{
	MPI_Attr_delete((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),keyval);
}

/*
 * Class:     mpi_Comm
 * Method:    Abort
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_Abort
  (JNIEnv *env, jobject jthis, jint errorcode)
{
  MPI_Abort((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)), errorcode);
}

/*
 * Class:     mpi_Comm
 * Method:    bsend
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;II)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_bsend
   (JNIEnv *env, jobject jthis, jobject buf, jint offset, 
	  jint count, jobject type, jint dest, jint tag)
{
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Bsend(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), dest, tag,
    (MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)));
  releaseBufPtr(env,buf,type,bufptr);
}

/*
 * Class:     mpi_Comm
 * Method:    ssend
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;II)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_ssend
   (JNIEnv *env, jobject jthis, jobject buf, jint offset, 
	  jint count, jobject type, jint dest, jint tag)
{
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Ssend(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), dest, tag,
    (MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)));
  releaseBufPtr(env,buf,type,bufptr);
}
/*
 * Class:     mpi_Comm
 * Method:    rsend
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;II)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_rsend
   (JNIEnv *env, jobject jthis, jobject buf, jint offset, 
	  jint count, jobject type, jint dest, jint tag)
{
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Rsend(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), dest, tag,
    (MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)));
  releaseBufPtr(env,buf,type,bufptr);
}
/*
 * Class:     mpi_Comm
 * Method:    Isend
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Request;)Lmpi/Request;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Isend
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Isend(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Ibsend
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Request;)Lmpi/Request;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Ibsend
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Ibsend(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Issend
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Request;)Lmpi/Request;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Issend
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Issend(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Irsend
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Request;)Lmpi/Request;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Irsend
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Irsend(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Irecv
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Status;)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Irecv
   (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint source, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Irecv(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	source, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);  
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Send_init
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Prequest;)Lmpi/Prequest;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Send_1init
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Send_init(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Bsend_init
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Prequest;)Lmpi/Prequest;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Bsend_1init
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Bsend_init(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Ssend_init
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Prequest;)Lmpi/Prequest;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Ssend_1init
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Ssend_init(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    Rsend_init
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Prequest;)Lmpi/Prequest;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Rsend_1init
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint dest, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Rsend_init(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	dest, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /*
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    recv_init
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;IILmpi/Prequest;)Lmpi/Prequest;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Recv_1init
  (JNIEnv *env, jobject jthis, jobject buf, jint offset, jint count, jobject
type, jint source, jint tag, jobject req)
{
  MPI_Request request;
  jbyte * bufptr=getBufPtr(env,buf,type);
  MPI_Recv_init(bufptr+offset,
    count, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	source, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&request);
  (*env)->SetLongField(env,req,reqhandleID,(jlong)request);
  releaseBufPtr(env,buf,type,bufptr);
  /* 
  if(request==MPI_REQUEST_NULL)
        return NULL;
  */
  return req;
}

/*
 * Class:     mpi_Comm
 * Method:    pack
 * Signature: (Ljava/lang/Object;IILmpi/Datatype;[BI)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_pack
  (JNIEnv *env, jobject jthis, jobject inbuf, jint offset, jint incount, jobject type, 
  jbyteArray outbuf, jint position)
{
  jboolean isCopy=JNI_TRUE;
  jbyte * ibufptr=getBufPtr(env,inbuf,type);
  jbyte * obufptr=(*env)->GetByteArrayElements(env,outbuf,&isCopy);
  int outsize=(*env)->GetArrayLength(env,outbuf);
  MPI_Pack(ibufptr+offset,
    incount, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	obufptr, outsize,(int*)&position,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)));
  releaseBufPtr(env,inbuf,type,ibufptr);
  (*env)->ReleaseByteArrayElements(env,outbuf,obufptr,0);
  return position;
}

/*
 * Class:     mpi_Comm
 * Method:    unpack
 * Signature: ([BILjava/lang/Object;IILmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_unpack
  (JNIEnv *env, jobject jthis, jbyteArray inbuf, jint position, jobject outbuf, jint offset,
  jint outcount, jobject type)
{
  jboolean isCopy=JNI_TRUE;
  jbyte * obufptr=getBufPtr(env,outbuf,type);
  jbyte * ibufptr=(*env)->GetByteArrayElements(env,inbuf,&isCopy);
  int insize=(*env)->GetArrayLength(env,inbuf);
  MPI_Unpack(ibufptr+offset,
    insize, (int*)&position,
	obufptr, outcount, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
	(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)));
  releaseBufPtr(env,outbuf,type,obufptr);
  (*env)->ReleaseByteArrayElements(env,inbuf,ibufptr,0);
  return position;
}

/*
 * Class:     mpi_Comm
 * Method:    Pack_size
 * Signature: (ILmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_Pack_1size
  (JNIEnv *env, jobject jthis, jint incount, jobject type)
{
   int size;
   MPI_Pack_size(incount, (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
		(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&size);
   return size;
}

/*
 * Class:     mpi_Comm
 * Method:    Iprobe
 * Signature: (II)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Iprobe
  (JNIEnv *env, jobject jthis, jint source, jint tag, jobject stat)
{
  MPI_Status status;
  int flag;
  MPI_Iprobe(source, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&flag,&status);
/*
  MPI_Get_count(&status, 
	(MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
    &status.count);
*/
  (*env)->SetIntField(env,stat, countID,status.count);
  (*env)->SetIntField(env,stat, sourceID,status.MPI_SOURCE);
  (*env)->SetIntField(env,stat, tagID,status.MPI_TAG);
  /* (*env)->SetIntField(env,stat, errorID,status.MPI_ERROR); */
  return stat;
}
/*
 * Class:     mpi_Comm
 * Method:    Probe
 * Signature: (II)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_Probe
  (JNIEnv *env, jobject jthis, jint source, jint tag,jobject stat)
{
int cnt;
  MPI_Status status;
  MPI_Probe(source, tag,(MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),&status);
/*
  MPI_Get_count(&status, 
	(MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)),
    &status.count);
*/
MPI_Get_count(&status,MPI_INT,&cnt);
  (*env)->SetIntField(env,stat, countID,status.count);
  (*env)->SetIntField(env,stat, sourceID,status.MPI_SOURCE);
  (*env)->SetIntField(env,stat, tagID,status.MPI_TAG);
  /* (*env)->SetIntField(env,stat, errorID,status.MPI_ERROR); */
  return stat;
}

/*
 * Class:     mpi_Comm
 * Method:    Errhandler_set
 * Signature: (Lmpi/Errhandler;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_Errhandler_1set
  (JNIEnv *env, jclass jthis, jobject errhandler)
{
	MPI_Errhandler_set((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),
		(MPI_Errhandler)((*env)->GetLongField(env,errhandler,ErrhandleID)));
}
/*
 * Class:     mpi_Comm
 * Method:    errorhandler_get
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_errorhandler_1get
  (JNIEnv *env, jclass jthis)
{
	MPI_Errhandler errhandler;
	MPI_Errhandler_get((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),
		&errhandler);
	return (jlong)errhandler;
}

/*
 * Class:     mpi_Comm
 * Method:    Topo_test
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_Topo_1test
  (JNIEnv *env, jobject jthis)
{
	int status;
	MPI_Topo_test((MPI_Comm)((*env)->GetLongField(env,jthis,CommhandleID)),
		&status);
	return status;
}

