/*
 * File         : .c
 * Headerfile   : .h 
 * Author       : Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.7 $
 * Updated      : $Date: 1998/04/09 18:39:57 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <stdio.h>
#include <string.h>
#include <mpi.h>
#include "mpi_MPI.h"

int len = 0;
char** sargs = 0;

jmethodID copyID, deleteID;
jclass comm_class;
jmethodID handleConstructorID;
JNIEnv *jenv;

/* extern GrouphandleID,reqhandleID; */

/*
 * Class:     mpi_MPI
 * Method:    InitNative
 * Signature: ([Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_mpi_MPI_InitNative
  (JNIEnv *env, jclass obj, jobjectArray argv)
{
  jsize i;
  jstring jc;
  len = (*env)->GetArrayLength(env,argv);
  sargs = (char**)calloc(len+1, sizeof(char*));
  for (i=0; i<len; i++) {
    jc=(jstring)(*env)->GetObjectArrayElement(env,argv,i);
    sargs[i]= (char*)calloc(strlen((*env)->GetStringUTFChars(env,jc,0))+1,
sizeof(char));
    strcpy(sargs[i],(*env)->GetStringUTFChars(env,jc,0));
  }

  MPI_Init(&len, &sargs);

  /*  printf("MPIInit...\n"); */
}                                                   

/*
 * Class:     mpi_MPI
 * Method:    Finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_MPI_Finalize
  (JNIEnv *env, jclass obj) {
  int i;

  /* printf("MPIFinalize...\n"); */

  MPI_Finalize();

  for (i=0; i<len; i++)
    free(sargs[i]);

  free(sargs);                            
}
                                             
/*
 * Class:     mpi_MPI
 * Method:    Get_processor_name
 * Signature: ([B)I
 */
JNIEXPORT jint JNICALL Java_mpi_MPI_Get_1processor_1name
  (JNIEnv *env, jclass obj, jbyteArray buf)
{
  int len;
  jboolean isCopy; 
  MPI_Get_processor_name((char*)((*env)->GetByteArrayElements(env,buf,&isCopy)),&len); 
  return len;
}

/*
 * Class:     mpi_MPI
 * Method:    Wtime
 * Signature: ()D
 */
JNIEXPORT jdouble JNICALL Java_mpi_MPI_Wtime
  (JNIEnv *env, jclass jthis)
{
	return MPI_Wtime();
}

/*
 * Class:     mpi_MPI
 * Method:    Wtick
 * Signature: ()D
 */
JNIEXPORT jdouble JNICALL Java_mpi_MPI_Wtick
  (JNIEnv *env, jclass jthis)
{
	return MPI_Wtick();
}

/*
 * Class:     mpi_MPI
 * Method:    Initialized
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_mpi_MPI_Initialized
  (JNIEnv *env, jclass jthis)
{
	int flag;
	MPI_Initialized(&flag);
	if(flag==0)
		return JNI_FALSE;
	else
		return JNI_TRUE;
}

/* !!! Pay attention to the Buffer attach and dettach because of the
//     java garbage collection.
//     Both the functions may be changed later.
*/

/*
 * Class:     mpi_MPI
 * Method:    Buffer_attach
 * Signature: ([B)V
 */
JNIEXPORT void JNICALL Java_mpi_MPI_Buffer_1attach
  (JNIEnv *env, jclass jthis, jbyteArray buf)
{
	jboolean isCopy;
    int size=(*env)->GetArrayLength(env,buf);
	MPI_Buffer_attach((*env)->GetByteArrayElements(env,buf,&isCopy),size); 
}

/*
 * Class:     mpi_MPI
 * Method:    Buffer_detach
 * Signature: ([B)V
 */
JNIEXPORT void JNICALL Java_mpi_MPI_Buffer_1detach
  (JNIEnv *env, jclass jthis, jbyteArray buf)
{
	jboolean isCopy;
	int size;
	MPI_Buffer_detach((*env)->GetByteArrayElements(env,buf,&isCopy),&size);
}

/*
 * Class:     mpi_MPI
 * Method:    Keyval_free
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_mpi_MPI_Keyval_1free
  (JNIEnv *env, jclass jthis, jint keyval)
{
    MPI_Keyval_free((int*)&keyval);
}

/* Java_copy_fn, Java_delete_fn are for function pointer in MPI_Keyval_create.
   Refer to P24 in document MPI_Java Interface.
   xli 1/29/98 
*/
int Java_copy_fn(MPI_Comm oldcomm, int keyval, void *extra_state, 
		void *attribute_val_in, void *attribute_val_out, int *flag)
{
    jobject joldcomm=(*jenv)->NewObject(jenv,comm_class,
		handleConstructorID, oldcomm);
	attribute_val_out = (*jenv)->CallObjectMethod(jenv,(jobject)extra_state,
			copyID,joldcomm, 
			keyval,(jobject)attribute_val_in);
	
	if(attribute_val_out)
		*flag=1;
	else *flag=0;
}

int Java_delete_fn(MPI_Comm oldcomm, int keyval, void *attribute_val,
		void *extra_state)
{
    jobject joldcomm=(*jenv)->NewObject(jenv,comm_class,
        handleConstructorID, oldcomm);
	(*jenv)->CallIntMethod(jenv,(jobject)extra_state,
            deleteID, joldcomm,keyval,(jobject)attribute_val);
}

/*
 * Class:     mpi_MPI
 * Method:    Keyval_create
 * Signature: (Lmpi/Call_back;)I
 */
JNIEXPORT jint JNICALL Java_mpi_MPI_Keyval_1create
  (JNIEnv *env, jclass jthis, jobject call_back)
{
	int keyval;
	jclass callback_class=(*env)->FindClass(env,"mpi/Call_back");
    copyID=(*env)->GetMethodID(env,callback_class,
        "copy","(Lmpi/Comm;I;Ljava/lang/Object;)Ljava/lang/Object");
    deleteID=(*env)->GetMethodID(env,callback_class,
        "delete","(Lmpi/Comm;I;Ljava/lang/Object;)V");

    comm_class=(*env)->FindClass(env,"mpi/Comm");
    handleConstructorID = (*env)->GetMethodID(env,
     comm_class, "<init>", "(I)V");

    jenv=env;
   
	MPI_Keyval_create(&Java_copy_fn,&Java_delete_fn, &keyval, 
		call_back);
	return keyval;
}

/*
 * Class:     mpi_MPI
 * Method:    SetConstant
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_MPI_SetConstant
  (JNIEnv *env, jclass jthis)
{
   jfieldID anysourceID=(*env)->GetStaticFieldID(env,jthis,"ANY_SOURCE","I");
   jfieldID anytagID=(*env)->GetStaticFieldID(env,jthis,"ANY_TAG","I");
   jfieldID procnullID=(*env)->GetStaticFieldID(env,jthis,"PROC_NULL","I");  
   jfieldID graphID=(*env)->GetStaticFieldID(env,jthis,"GRAPH","I");
   jfieldID cartID=(*env)->GetStaticFieldID(env,jthis,"CART","I");
   jfieldID bsendoverID=(*env)->GetStaticFieldID(env,jthis,"BSEND_OVERHEAD","I");
   jfieldID undefinedID=(*env)->GetStaticFieldID(env,jthis,"UNDEFINED","I");
   jfieldID identID=(*env)->GetStaticFieldID(env,jthis,"IDENT","I");
   jfieldID congruentID=(*env)->GetStaticFieldID(env,jthis,"CONGRUENT","I");
   jfieldID similarID=(*env)->GetStaticFieldID(env,jthis,"SIMILAR","I");
   jfieldID unequalID=(*env)->GetStaticFieldID(env,jthis,"UNEQUAL","I");
   jfieldID tagubID=(*env)->GetStaticFieldID(env,jthis,"TAG_UB","I");
   jfieldID hostID=(*env)->GetStaticFieldID(env,jthis,"HOST","I");
   jfieldID ioID=(*env)->GetStaticFieldID(env,jthis,"IO","I");

   (*env)->SetStaticIntField(env,jthis,anysourceID,MPI_ANY_SOURCE);
   (*env)->SetStaticIntField(env,jthis,anytagID,MPI_ANY_TAG);
   (*env)->SetStaticIntField(env,jthis,procnullID,MPI_PROC_NULL);
   (*env)->SetStaticIntField(env,jthis,graphID,MPI_GRAPH);
   (*env)->SetStaticIntField(env,jthis,cartID,MPI_CART);
   (*env)->SetStaticIntField(env,jthis,bsendoverID,MPI_BSEND_OVERHEAD);
   (*env)->SetStaticIntField(env,jthis,undefinedID,MPI_UNDEFINED);
   (*env)->SetStaticIntField(env,jthis,identID,MPI_IDENT);
   (*env)->SetStaticIntField(env,jthis,congruentID,MPI_CONGRUENT);
   (*env)->SetStaticIntField(env,jthis,similarID,MPI_SIMILAR);
   (*env)->SetStaticIntField(env,jthis,unequalID,MPI_UNEQUAL);
   (*env)->SetStaticIntField(env,jthis,tagubID,MPI_TAG_UB);
   (*env)->SetStaticIntField(env,jthis,hostID,MPI_HOST);
   (*env)->SetStaticIntField(env,jthis,ioID,MPI_IO);
}
