/*
 * File         : mpi_Status.c
 * Headerfile   : mpi_Status.h 
 * Author       : Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.6 $
 * Updated      : $Date: 1998/04/16 16:17:29 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <mpi.h>
#include "mpi_Status.h"

extern jfieldID DatatypehandleID;
jfieldID sourceID,tagID,countID, indexID;
/* jfiedldID errorID; */

/*
 * Class:     mpi_Status
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Status_init
  (JNIEnv *env, jclass jthis)
{
  sourceID=(*env)->GetFieldID(env,jthis,"source","I");
  tagID=(*env)->GetFieldID(env,jthis,"tag","I");
  countID=(*env)->GetFieldID(env,jthis,"count","I");
  indexID=(*env)->GetFieldID(env,jthis,"index","I");
  /* errorID=(*env)->GetFieldID(env,jthis,"error","I"); */
}

/*
 * Class:     mpi_Status
 * Method:    Get_count
 * Signature: (Lmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_Get_1count
  (JNIEnv *env, jobject jthis, jobject type)
{
   int count;

   MPI_Status stat;
   stat.count=(*env)->GetIntField(env,jthis,countID);
   stat.MPI_SOURCE=(*env)->GetIntField(env,jthis,sourceID);
   stat.MPI_TAG=(*env)->GetIntField(env,jthis,tagID);
   /* stat.MPI_ERROR=(*env)->GetIntField(env,jthis,errorID); */
/* printf("%d\n", stat.count); */
   MPI_Get_count(&stat,
	(MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), &count);
   return count;

/*   return ((*env)->GetIntField(env,jthis,countID)); */
}

/*
 * Class:     mpi_Status
 * Method:    Test_cancelled
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_mpi_Status_Test_1cancelled
  (JNIEnv *env, jobject jthis)
{
   int flag;
   MPI_Status stat;
   stat.count=(*env)->GetIntField(env,jthis,countID);
   stat.MPI_SOURCE=(*env)->GetIntField(env,jthis,sourceID);
   stat.MPI_TAG=(*env)->GetIntField(env,jthis,tagID);
   /* stat.MPI_ERROR=(*env)->GetIntField(env,jthis,errorID); */
   MPI_Test_cancelled(&stat,&flag);
   if(flag==0)
        return JNI_FALSE;
    else
        return JNI_TRUE;
}

/*
 * Class:     mpi_Status
 * Method:    Get_elements
 * Signature: (Lmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_Get_1elements
  (JNIEnv *env, jobject jthis, jobject type)       
{
   int count;
   MPI_Status stat;
   stat.count=(*env)->GetIntField(env,jthis,countID);
   stat.MPI_SOURCE=(*env)->GetIntField(env,jthis,sourceID);
   stat.MPI_TAG=(*env)->GetIntField(env,jthis,tagID);
   /* stat.MPI_ERROR=(*env)->GetIntField(env,jthis,errorID);*/
   MPI_Get_elements(&stat,
    (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), &count);
   return count;
}

